# -*- coding: utf-8 -*-


from .singleton import Singleton as dpSingleton
from .loader import Loader as dpLoader


class EngineSingleton(dpSingleton):
    @property
    def options(self):
        if not hasattr(self, '_options'):
            import tornado.options

            class TornadoOptions(object):
                _options = tornado.options.options

                def __getattr__(self, item):
                    try:
                        return self._options.__getattr__(item)
                    except AttributeError:
                        return None

            self._options = TornadoOptions()
        
        try:
            return self._options
        except AttributeError:
            return None

    @property
    def config(self):
        if not hasattr(self, '_config'):
            self._config = dpLoader('config')

        return self._config

    @property
    def model(self):
        if not hasattr(self, '_model'):
            self._model = dpLoader('model')

        return self._model

    @property
    def helper(self):
        if not hasattr(self, '_helper'):
            self._helper = dpLoader('helper')

        return self._helper

    @property
    def view(self):
        if not hasattr(self, '_view'):
            from .view import View as dpView
            self._view = dpView()

        return self._view

    @property
    def m17n(self):
        if not hasattr(self, '_m17n'):
            from .m17n import M17n as dpM17n
            self._m17n = dpM17n()

        return self._m17n

    @property
    def cache(self):
        if not hasattr(self, '_cache'):
            from .cache import Cache as dpCache
            self._cache = dpCache()

        return self._cache

    @property
    def logger(self):
        if not hasattr(self, '_logger'):
            from .logger import Logger as dpLogger
            self._logger = dpLogger(self)

        return self._logger

    @property
    def vars(self):
        if not hasattr(self, '_vars'):
            from .variable import Variable as dpVariable
            self._vars = dpVariable(self)

        return self._vars

    @property
    def executor(self):
        if not hasattr(self, '_executor'):
            if self.options.max_worker is None:
                return None

            import tornado.concurrent
            import tornado.options

            if self.options.max_worker:
                self._executor = tornado.concurrent.futures.ThreadPoolExecutor(self.options.max_worker)
            else:
                self._executor = None

        return self._executor


class Engine(object):
    @property
    def options(self):
        return EngineSingleton().options

    @property
    def config(self):
        return EngineSingleton().config

    @property
    def model(self):
        return EngineSingleton().model

    @property
    def helper(self):
        return EngineSingleton().helper

    @property
    def view(self):
        return EngineSingleton().view

    @property
    def m17n(self):
        return EngineSingleton().m17n

    @property
    def cache(self):
        return EngineSingleton().cache

    @property
    def logger(self):
        return EngineSingleton().logger

    @property
    def logging(self):
        return self.logger

    @property
    def vars(self):
        return EngineSingleton().vars

    @property
    def _executor(self):
        return EngineSingleton().executor



# -*- coding: utf-8 -*-


import zipfile

from dp_tornado.engine.helper import Helper as dpHelper


class ZipFile(object):
    def __init__(self, file, mode="r", compression=zipfile.ZIP_STORED, allowZip64=False):
        self.zipfile = zipfile.ZipFile(file=file, mode=mode, compression=compression, allowZip64=allowZip64)

    def appends(self, files, compress_type=None):
        for file in files:
            if isinstance(file, (tuple, list)):
                filename = file[0]
                arcname = file[1]
            else:
                filename = file
                arcname = None

            self.zipfile.write(filename=filename, arcname=arcname, compress_type=compress_type)

        return True

    def close(self):
        return self.zipfile.close()


class ZipHelper(dpHelper):
    def new(self, file, mode="r", compression=zipfile.ZIP_STORED, allowZip64=False):
        return ZipFile(file, mode, compression, allowZip64)



# -*- coding: utf-8 -*-


from dp_tornado.engine.helper import Helper as dpHelper


class EmailHelper(dpHelper):
    def validate(self, e):
        return self.helper.validator.email.validate(e)

    def username_from_email(self, email):
        pos = email.find('@')

        if pos != -1:
            return email[:pos]
        else:
            return None

    def check_username_length_from_email(self, email, length):
        username = self.username_from_email(email)

        if username:
            return True if len(username) <= length else False
        else:
            return False


# -*- coding: utf-8 -*-


import sys
import importlib


if __name__ == '__main__':
    if len(sys.argv) <= 2:
        exit(0)

    runner = None
    app_path = sys.argv[1]
    module = sys.argv[2]
    timeout = sys.argv[3] if len(sys.argv) >= 4 else None

    sys.path.append(app_path)

    try:
        module = importlib.import_module(module)

    except ImportError as e:
        print(e)
        print('The specified scheduler module is invalid. (%s)' % module)
        exit(1)

    try:
        runner = getattr(module, 'Scheduler')(timeout)

    except AttributeError:
        print('The specified scheduler module is not implemented. (%s)' % module)
        exit(1)

    try:
        runner.run()

    except Exception as e:
        import traceback
        traceback.print_exc()
